namespace SourceCode.CustomActions
{
	using System;
	using System.Data;
	using System.Drawing;
	using System.Web;
	using System.Web.UI.WebControls;
	using System.Web.UI.HtmlControls;
	using System.Resources;
	using System.Globalization;

	using SourceCode.K2Mng;
	using SourceCode.SharePointWorkflowServicesWebObject;
	using SourceCode.SharePointWorkflowServices2005.ConfigObject;

	/// <summary>
	///		Summary description for K2StartProcess.
	/// </summary>
	public class K2StartProcess : UCBase
	{
		#region - Controls -
		protected System.Web.UI.WebControls.Label lblPageDescription;
		protected System.Web.UI.WebControls.Button btnCancel;
		protected System.Web.UI.WebControls.Button btnOK;
		protected System.Web.UI.WebControls.Button btnBack;
		protected System.Web.UI.WebControls.TextBox txtUserNote;
		protected System.Web.UI.WebControls.Label lblNotes;
		protected System.Web.UI.WebControls.Label lblAdditionalSettingsInstructions;
		protected System.Web.UI.WebControls.Label lblAdditionalSettings;
		protected System.Web.UI.WebControls.Label lblProcessDetailsInstructions;
		protected System.Web.UI.WebControls.DropDownList ddlProcesses;
		protected System.Web.UI.WebControls.Label lblProcesses;
		protected System.Web.UI.WebControls.Button btnConnect;
		protected System.Web.UI.WebControls.Label lblK2Server;
		protected System.Web.UI.WebControls.TextBox txtK2Server;
		protected System.Web.UI.WebControls.CustomValidator cvErrors;
		protected System.Web.UI.WebControls.Label lblProcessDetails;
		#endregion

		#region - Variables -
		public string DefaultButton = "";
		public string BackButton = "";
		#endregion

		#region - Page Load -
		private void Page_Load(object sender, System.EventArgs e)
		{
			// set the existing resourcemanager to use our resource
			ConfigWFRM = new ResourceManager("SourceCode.CustomActions.Resources.CustomActionsLabels", System.Reflection.Assembly.GetExecutingAssembly());
		}
		#endregion

		#region - OnPreRender -
		protected override void OnPreRender(EventArgs e)
		{
			base.OnPreRender (e);

			SetupControls();
			LoadData();
		}
		#endregion

		#region ~ Data Manipulation ~

		#region - LoadData -
		private void LoadData()
		{
			try
			{
				if (ucCWFData.myState["NewAction"] == "false" &&
					ucCWFData.currentAction != null &&
					ucCWFData.currentAction.SerializedXML != "")
				{
					ProcessDetails _procDetails = ProcessDetails.GetProcess(ucCWFData.currentAction.SerializedXML);

					txtK2Server.Text = _procDetails.K2ServerName;
					
					FillProcessList();

					ddlProcesses.Items.FindByText(_procDetails.FullName).Selected = true;

					// since we encode our users note, we need to decode it
					// as well to display correctly
					txtUserNote.Text = System.Xml.XmlConvert.DecodeName(ucCWFData.currentAction.UserNote);
				}
				else
				{
					txtK2Server.Text = Common.GetK2Server;
					FillProcessList();
					txtUserNote.Text = "";
				}
			}
			catch (Exception ex)
			{
				cvErrors.ErrorMessage = ex.Message;
				cvErrors.IsValid = false;
			}
		}
		#endregion

		#region - GetProcessXml -
		private string GetProcessXml()
		{
			ProcessDetails _process = new ProcessDetails();
			string _procFolder = "";
			string _procName = "";

			GetProcessFolderAndName(ref _procFolder, ref _procName);

			_process.K2ServerName = txtK2Server.Text;
			_process.Port = 5252;
			_process.Folder = _procFolder;
			_process.Name = _procName;
			_process.FullName = ddlProcesses.SelectedItem.Text;

			return _process.GetXML();
		}
		#endregion

		#endregion

		#region ~ Helper Methods ~

		#region - CheckFields -
		private bool CheckFields()
		{
			System.Text.RegularExpressions.Regex _regEx = new System.Text.RegularExpressions.Regex(Common.DefaultRegEx);
			
			if (txtUserNote.Text.Length > 0 && !_regEx.IsMatch(txtUserNote.Text))
			{
				string _msg = ConfigWFRM.GetString("ErrorInvalidCharacters", ci).Replace("{0}", lblNotes.Text);
				Page.RegisterClientScriptBlock("", "<script language='javascript'>alert('" + _msg + "');</script>");
				return false;
			}

			if (txtUserNote.Text.Length > 8000)
			{
				string _msg = ConfigWFRM.GetString("ErrorTextExceedingCharLimit8000", ci).Replace("{0}", lblNotes.Text);
				Page.RegisterClientScriptBlock("", "<script language='javascript'>alert('" + _msg + "');</script>");
				return false;
			}

			return true;
		}
		#endregion

		#region - SetupControls -
		private void SetupControls()
		{
			DefaultButton = "";
			BackButton = btnCancel.UniqueID.Replace(":", "_");
			// sets all the labels on the page with the correct
			// values from the resource file
			UICommon.setLabelValues(this, ConfigWFRM, ci);
			// sets the page's description - telling the user what the page is used for
			lblPageDescription.Text = ConfigWFRM.GetString("lblK2StartProcess_PageDescription", ci);
		}
		#endregion

		#region - FillProcessList -
		private void FillProcessList()
		{
			K2Manager _k2Mngr = new K2Manager();

			try
			{
				_k2Mngr.Login(txtK2Server.Text, 5252);
				ProcessSets _procSets = _k2Mngr.GetProcSets();

				ddlProcesses.Items.Clear();

				foreach (ProcessSet _procSet in _procSets)
				{
					if (_procSet.Folder != "SharePoint Workflow Services")
					{
						ddlProcesses.Items.Add(new ListItem(_procSet.FullName, _procSet.ProcSetID.ToString()));
					}
				}

				if (ddlProcesses.Items.Count == 0)
				{
					ddlProcesses.Items.Add("None Found");
				}
				else
				{
					ddlProcesses.Items.Insert(0, new ListItem(" - Select - ", "-1"));
				}
			}
			catch (Exception ex)
			{
				cvErrors.ErrorMessage = ex.Message;
				cvErrors.IsValid = false;
			}
			finally
			{
				_k2Mngr.Logout();
			}
		}
		#endregion

		#region - GetProcessFolderAndName -
		private void GetProcessFolderAndName(ref string folder, ref string name)
		{
			string[] _temp = ddlProcesses.SelectedItem.Text.Split('\\');

			folder = _temp.GetValue(0).ToString();
			name = _temp.GetValue(1).ToString();
		}
		#endregion

		#endregion

		#region ~ Control Events ~

		#region - btnConnect_Click -
		private void btnConnect_Click(object sender, System.EventArgs e)
		{
			FillProcessList();
		}
		#endregion

		#region - btnBack_Click -
		private void btnBack_Click(object sender, System.EventArgs e)
		{
			if (ucCWFData.myState["NewAction"] == "true")
			{
				ucCWFData.ucPage = ucCWFData.ucBackPage;
			}
			else
			{
				ucCWFData.ucPage = ucCWFData.myState["BackPanel"];
			}
			MoveNextPage();
		}
		#endregion

		#region - btnOK_Click -
		private void btnOK_Click(object sender, System.EventArgs e)
		{
			// add or update the action
			// if you are going to allow special characters in the users note, 
			// you need to encode the string using - System.Xml.XmlConvert.EncodeName('')
			// if not, then it can be removed
			AddOrUpdateAction(ucCWFData.myState["ActionBase"], System.Xml.XmlConvert.EncodeName(txtUserNote.Text), GetProcessXml());
			SaveTheState();
			ucCWFData.ucPage = ucCWFData.myState["BackPanel"];
			MoveNextPage();
		}
		#endregion

		#region - btnCancel_Click -
		private void btnCancel_Click(object sender, System.EventArgs e)
		{
			ucCWFData.ucPage = ucCWFData.myState["BackPanel"];
			MoveNextPage();
		}
		#endregion

		#endregion

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		///		Required method for Designer support - do not modify
		///		the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.btnConnect.Click += new System.EventHandler(this.btnConnect_Click);
			this.btnBack.Click += new System.EventHandler(this.btnBack_Click);
			this.btnOK.Click += new System.EventHandler(this.btnOK_Click);
			this.btnCancel.Click += new System.EventHandler(this.btnCancel_Click);
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion
	}
}
