namespace SourceCode.CustomActions
{
	using System;
	using System.Data;
	using System.Drawing;
	using System.Web;
	using System.Web.UI.WebControls;
	using System.Web.UI.HtmlControls;
	using System.Resources;
	using System.Globalization;

	// These are required references
	using SourceCode.SharePointWorkflowServicesWebObject;
	using SourceCode.SharePointWorkflowServices2005.ConfigObject;

	/// <summary>
	///	Template Custom Action User Control
	///	Demonstrates the required references, variables, methods, etc.
	/// </summary>
	// NOTE: The user control must inherit from UCBase
	public class TemplateActionUserControl : UCBase
	{
		#region - Controls -
		protected System.Web.UI.WebControls.Button btnCancel;
		protected System.Web.UI.WebControls.Button btnOK;
		protected System.Web.UI.WebControls.Button btnBack;
		protected System.Web.UI.WebControls.TextBox txtUserNote;
		protected System.Web.UI.WebControls.Label lblNotes;
		protected System.Web.UI.WebControls.Label lblAdditionalSettingsInstructions;
		protected System.Web.UI.WebControls.Label lblAdditionalSettings;
		protected System.Web.UI.WebControls.DropDownList ddlDummyDropDown;
		protected System.Web.UI.WebControls.Label lblDummyLabel2;
		protected System.Web.UI.WebControls.TextBox txtDummyTextBox;
		protected System.Web.UI.WebControls.Label lblDummyLabel;
		protected System.Web.UI.WebControls.Label lblInstructions;
		protected System.Web.UI.WebControls.Label lblTitle;
		protected System.Web.UI.WebControls.CustomValidator cvErrors;
		protected System.Web.UI.WebControls.Label lblPageDescription;
		#endregion

		#region - Constants -
		// REQUIRED 
		// this is required if you want to allow special characters in the usernote
		public readonly string DefaultRegEx = @"^[a-zA-Z0-9]*[a-zA-Z0-9\s`~!@#$%^&*\(\)\-_=+\[{\]};:'"",<.>\/?\\|]+$";
		#endregion

		#region - Variables -
		// REQUIRED
		public string DefaultButton = "";
		// REQUIRED
		public string BackButton = "";
		#endregion

		#region - Page Load -
		private void Page_Load(object sender, System.EventArgs e)
		{
			// required
			// set the inherited resource manager to use the resource file for this custom action
			ConfigWFRM = new ResourceManager("SourceCode.CustomActions.Resources.CustomActionsLabels", System.Reflection.Assembly.GetExecutingAssembly());
		}
		#endregion

		#region - OnPreRender -
		// this method is create by typing: override OnPreRender, and pressing TAB
		protected override void OnPreRender(EventArgs e)
		{
			base.OnPreRender (e);

			// REQRUIRED
			SetupControls();
			// REQRUIRED
			LoadData();
		}
		#endregion

		#region - SetupControls -
		// REQUIRED
		private void SetupControls()
		{
			// can use this to catch the <Enter> key when pressed,
			// and fire a desired event (eg. the OK button)
			DefaultButton = "";
			// captures the <Esc> key when pressed, and fires the cancel button's event
			BackButton = btnCancel.UniqueID.Replace(":", "_");
			// sets all the labels on the page with the correct
			// values from the resource file
			UICommon.setLabelValues(this, ConfigWFRM, ci);
			// sets the page's description - telling the user what the page is used for
			lblPageDescription.Text = ConfigWFRM.GetString("lblK2StartProcess_PageDescription", ci);
		}
		#endregion

		#region - LoadData -
		// REQUIRED
		private void LoadData()
		{
			try
			{
				// this will check if the action has been added, or if this is 
				// a new action being added.
				if (ucCWFData.myState["NewAction"] == "false" &&
					ucCWFData.currentAction != null &&
					ucCWFData.currentAction.SerializedXML != "")
				{
				}
				else
				{
					// set our dummy textbox with data
					txtDummyTextBox.Text = "";
					// REQUIRED: clear the user note
					txtUserNote.Text = "";
				}
			}
			catch (Exception ex)
			{
				cvErrors.ErrorMessage = ex.Message;
				cvErrors.IsValid = false;
			}
		}
		#endregion

		#region - GetProcessXml -
		// REQUIRED
		// this method is used to capture the xml that will be serialized
		// and saved to the configured workflow data.
		private string GetProcessXml()
		{
			// a seperate class that represents the object's data being captured can 
			// be created, and methods for generating the xml from the object, and generating 
			// an object from the serialized xml can be added to the class
			// Example: 
			// Class _object = new Class();
			// _object.Title = txtTitle.Text;
			// _object.Description = txtDescription.Text;
			// return _object.GetXML();		<-- this is the method which is added to the class to generate the XML

			// another option is to generate the xml within this class

			return "";
		}
		#endregion

		#region - CheckFields -
		// REQUIRED
		// this is used to check that the user note contains only specific special characters
		private bool CheckFields()
		{
			System.Text.RegularExpressions.Regex _regEx = new System.Text.RegularExpressions.Regex(DefaultRegEx);
			
			if (txtUserNote.Text.Length > 0 && !_regEx.IsMatch(txtUserNote.Text))
			{
				string _msg = ConfigWFRM.GetString("ErrorInvalidCharacters", ci).Replace("{0}", lblNotes.Text);
				Page.RegisterClientScriptBlock("", "<script language='javascript'>alert('" + _msg + "');</script>");
				return false;
			}

			if (txtUserNote.Text.Length > 8000)
			{
				string _msg = ConfigWFRM.GetString("ErrorTextExceedingCharLimit8000", ci).Replace("{0}", lblNotes.Text);
				Page.RegisterClientScriptBlock("", "<script language='javascript'>alert('" + _msg + "');</script>");
				return false;
			}

			return true;
		}
		#endregion

		#region - btnBack_Click -
		// REQUIRED - AS IS!!
		private void btnBack_Click(object sender, System.EventArgs e)
		{
			if (ucCWFData.myState["NewAction"] == "true")
			{
				ucCWFData.ucPage = ucCWFData.ucBackPage;
			}
			else
			{
				ucCWFData.ucPage = ucCWFData.myState["BackPanel"];
			}
			MoveNextPage();
		}
		#endregion

		#region - btnOK_Click -
		// REQUIRED
		// every line of code below is needed!!!
		private void btnOK_Click(object sender, System.EventArgs e)
		{
			// add or update the action 
			AddOrUpdateAction(ucCWFData.myState["ActionBase"], System.Xml.XmlConvert.EncodeName(txtUserNote.Text), GetProcessXml());
			// this call saves the state data
			SaveTheState();
			// sets the panel to be displayed when the action has been saved
			ucCWFData.ucPage = ucCWFData.myState["BackPanel"];
			// hides the current panel and sets the panel above to visible
			MoveNextPage();
		}
		#endregion

		#region - btnCancel_Click -
		// REQUIRED - AS IS!!
		private void btnCancel_Click(object sender, System.EventArgs e)
		{
			ucCWFData.ucPage = ucCWFData.myState["BackPanel"];
			MoveNextPage();
		}
		#endregion

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		///		Required method for Designer support - do not modify
		///		the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.btnBack.Click += new System.EventHandler(this.btnBack_Click);
			this.btnOK.Click += new System.EventHandler(this.btnOK_Click);
			this.btnCancel.Click += new System.EventHandler(this.btnCancel_Click);
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion
	}
}
